package com.sun.org.apache.xalan.internal.xsltc.compiler;

import com.sun.org.apache.bcel.internal.generic.InstructionList;
import com.sun.org.apache.bcel.internal.generic.ConstantPoolGen;
import com.sun.org.apache.bcel.internal.generic.INVOKESTATIC;
import com.sun.org.apache.bcel.internal.generic.PUSH;
import com.sun.org.apache.xalan.internal.xsltc.compiler.util.MethodGenerator;
import com.sun.org.apache.xalan.internal.xsltc.compiler.util.ClassGenerator;
import com.sun.org.apache.xalan.internal.xsltc.compiler.util.TypeCheckError;
import com.sun.org.apache.xalan.internal.xsltc.compiler.util.Type;
import java.util.List;
import com.sun.org.apache.xalan.internal.xsltc.compiler.util.Util;
import com.sun.org.apache.xalan.internal.xsltc.compiler.util.ErrorMsg;
import java.util.Vector;

final class UnsupportedElement extends SyntaxTreeNode
{
    private Vector _fallbacks;
    private ErrorMsg _message;
    private boolean _isExtension;
    
    public UnsupportedElement(String uri, String prefix, String local, boolean isExtension) {
        super(uri, prefix, local);
        this._fallbacks = null;
        this._message = null;
        this._isExtension = false;
        this._isExtension = isExtension;
    }
    
    public void setErrorMessage(ErrorMsg message) {
        this._message = message;
    }
    
    @Override
    public void display(int indent) {
        this.indent(indent);
        Util.println("Unsupported element = " + this._qname.getNamespace() + ":" + this._qname.getLocalPart());
        this.displayContents(indent + 4);
    }
    
    private void processFallbacks(Parser parser) {
        List<SyntaxTreeNode> children = this.getContents();
        if (children != null) {
            for (int count = children.size(), i = 0; i < count; ++i) {
                SyntaxTreeNode child = (SyntaxTreeNode)children.get(i);
                if ((child instanceof Fallback)) {
                    Fallback fallback = (Fallback)child;
                    fallback.activate();
                    fallback.parseContents(parser);
                    if (this._fallbacks == null) {
                        this._fallbacks = new Vector();
                    }
                    this._fallbacks.addElement(child);
                }
            }
        }
    }
    
    @Override
    public void parseContents(Parser parser) {
        this.processFallbacks(parser);
    }
    
    @Override
    public Type typeCheck(SymbolTable stable) throws TypeCheckError {
        if (this._fallbacks != null) {
            for (int count = this._fallbacks.size(), i = 0; i < count; ++i) {
                Fallback fallback = (Fallback)this._fallbacks.elementAt(i);
                fallback.typeCheck(stable);
            }
        }
        return Type.Void;
    }
    
    @Override
    public void translate(ClassGenerator classGen, MethodGenerator methodGen) {
        if (this._fallbacks != null) {
            for (int count = this._fallbacks.size(), i = 0; i < count; ++i) {
                Fallback fallback = (Fallback)this._fallbacks.elementAt(i);
                fallback.translate(classGen, methodGen);
            }
        }
        else {
            ConstantPoolGen cpg = classGen.getConstantPool();
            InstructionList il = methodGen.getInstructionList();
            int unsupportedElem = cpg.addMethodref("com.sun.org.apache.xalan.internal.xsltc.runtime.BasisLibrary", "unsupported_ElementF", "(Ljava/lang/String;Z)V");
            il.append(new PUSH(cpg, this.getQName().toString()));
            il.append(new PUSH(cpg, this._isExtension));
            il.append(new INVOKESTATIC(unsupportedElem));
        }
    }
}
